/* @font-face {
  font-family: 'Ubuntu Mono derivative Powerline';
  src: url('./Ubuntu Mono derivative Powerline.ttf') format('truetype');
  font-weight: normal;
  font-style: normal;
} */

@font-face {
  font-family: 'ChineseFont';
  src: url('./Ubuntu Mono derivative Powerline.ttf') format('truetype');
  /* unicode-range: U+4E00-9FFF; */
}

@font-face {
  font-family: 'ChineseFontBold';
  src: url('./Ubuntu Mono derivative Powerline Bold.ttf') format('truetype');
  /* unicode-range: U+4E00-9FFF; */
}

/* 
@font-face {
  font-family: 'EnglishFont';
  src: local('Arial'), local('Helvetica');
  unicode-range: U+0000-007F; 
} 
*/


body {
  margin: 0;
  font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', 'Roboto', 'Oxygen',
    'Ubuntu', 'Cantarell', 'Fira Sans', 'Droid Sans', 'Helvetica Neue',
    sans-serif;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

code {
  font-family: source-code-pro, Menlo, Monaco, Consolas, 'Courier New',
    monospace;
}
